@@ -1,13 +1,10 @@
-// tslint:disable:no-submodule-imports
 import * as fs from 'fs'
 import * as glob from 'glob'
 import * as Path from 'path'
-
 import * as request from 'request-promise-native'
-import * as Parser from 'tree-sitter'
-import * as bash from 'tree-sitter-bash'
 import * as URI from 'urijs'
 import * as LSP from 'vscode-languageserver'
+import * as Parser from 'web-tree-sitter'
 
 import { uniqueBasedOnHash } from './util/array'
 import { flattenArray, flattenObjectValues } from './util/flatten'
@@ -33,14 +30,19 @@ export default class Analyzer {
    * If the rootPath is provided it will initialize all *.sh files it can find
    * anywhere on that path.
    */
-  public static fromRoot(
-    connection: LSP.Connection,
-    rootPath: string | null,
-  ): Promise<Analyzer> {
+  public static fromRoot({
+    connection,
+    rootPath,
+    parser,
+  }: {
+    connection: LSP.Connection
+    rootPath: string | null
+    parser: Parser
+  }): Promise<Analyzer> {
     // This happens if the users opens a single bash script without having the
     // 'window' associated with a specific project.
     if (!rootPath) {
-      return Promise.resolve(new Analyzer())
+      return Promise.resolve(new Analyzer(parser))
     }
 
     return new Promise((resolve, reject) => {
@@ -48,13 +50,13 @@ export default class Analyzer {
         if (err != null) {
           reject(err)
         } else {
-          const analyzer = new Analyzer()
+          const analyzer = new Analyzer(parser)
           paths.forEach(p => {
             const absolute = Path.join(rootPath, p)
             // only analyze files, glob pattern may match directories
             if (fs.existsSync(absolute) && fs.lstatSync(absolute).isFile()) {
-              const uri = 'file://' + absolute
-              connection.console.log('Analyzing ' + uri)
+              const uri = `file://${absolute}`
+              connection.console.log(`Analyzing ${uri}`)
               analyzer.analyze(
                 uri,
                 LSP.TextDocument.create(
@@ -72,6 +74,8 @@ export default class Analyzer {
     })
   }
 
+  private parser: Parser
+
   private uriToTextDocument: { [uri: string]: LSP.TextDocument } = {}
 
   private uriToTreeSitterTrees: Trees = {}
@@ -83,9 +87,15 @@ export default class Analyzer {
 
   private treeSitterTypeToLSPKind: Kinds = {
     // These keys are using underscores as that's the naming convention in tree-sitter.
+    /* eslint-disable @typescript-eslint/camelcase */
     environment_variable_assignment: LSP.SymbolKind.Variable,
     function_definition: LSP.SymbolKind.Function,
     variable_assignment: LSP.SymbolKind.Variable,
+    /* eslint-enable @typescript-eslint/camelcase */
+  }
+
+  public constructor(parser: Parser) {
+    this.parser = parser
   }
 
   /**
@@ -126,6 +136,7 @@ export default class Analyzer {
       interestingNode.endIndex,
     )
 
+    // FIXME: type the response and unit test it
     const explainshellResponse = await request({
       uri: URI(endpoint)
         .path('/api/explain')
@@ -148,7 +159,7 @@ export default class Analyzer {
         interestingNode.startIndex
 
       const match = explainshellResponse.matches.find(
-        helpItem =>
+        (helpItem: any) =>
           helpItem.start <= offsetOfMousePointerInCommand &&
           offsetOfMousePointerInCommand < helpItem.end,
       )
@@ -179,7 +190,7 @@ export default class Analyzer {
     const tree = this.uriToTreeSitterTrees[uri]
     const contents = this.uriToFileContent[uri]
 
-    const locations = []
+    const locations: LSP.Location[] = []
 
     TreeSitterUtil.forEach(tree.rootNode, n => {
       let name: string = null
@@ -215,7 +226,7 @@ export default class Analyzer {
    * Find unique symbol completions for the given file.
    */
   public findSymbolCompletions(uri: string): LSP.CompletionItem[] {
-    const hashFunction = ({ name, kind }) => `${name}${kind}`
+    const hashFunction = ({ name, kind }: LSP.SymbolInformation) => `${name}${kind}`
 
     return uniqueBasedOnHash(this.findSymbols(uri), hashFunction).map(
       (symbol: LSP.SymbolInformation) => ({
@@ -239,16 +250,14 @@ export default class Analyzer {
   public analyze(uri: string, document: LSP.TextDocument): LSP.Diagnostic[] {
     const contents = document.getText()
 
-    const parser = new Parser()
-    parser.setLanguage(bash)
-    const tree = parser.parse(contents)
+    const tree = this.parser.parse(contents)
 
     this.uriToTextDocument[uri] = document
     this.uriToTreeSitterTrees[uri] = tree
     this.uriToDeclarations[uri] = {}
     this.uriToFileContent[uri] = contents
 
-    const problems = []
+    const problems: LSP.Diagnostic[] = []
 
     TreeSitterUtil.forEach(tree.rootNode, (n: Parser.SyntaxNode) => {
       if (n.type === 'ERROR') {
